@@ -30,8 +30,10 @@ LANGS = {
     "es": "es_core_news_sm",  # Spanish
     "fi": "fi_core_news_sm",  # Finnish
     "fr": "fr_core_news_sm",  # French
+    "hr": "hr_core_news_sm",  # Croatian
     "it": "it_core_news_sm",  # Italian
     "ja": "ja_core_news_sm",  # Japanese
+    "ko": "ko_core_news_sm",  # Korean
     "lt": "lt_core_news_sm",  # Lithuanian
     "mk": "mk_core_news_sm",  # Macedonian
     "nb": "nb_core_news_sm",  # Norwegian
@@ -41,6 +43,7 @@ LANGS = {
     "ro": "ro_core_news_sm",  # Romanian
     "ru": "ru_core_news_sm",  # Russian
     "sv": "sv_core_news_sm",  # Swedish
+    "uk": "uk_core_news_sm",  # Ukrainian
     "zh": "zh_core_web_sm",  # Chinese
 }
 
@@ -102,7 +105,7 @@ class Corpus:
     @memoize
     def _nlp(self, lang: str) -> pd.Series:
         try:
-            nlp = spacy.load(lang, disable=["ner", "textcat"])
+            nlp = spacy.load(lang, exclude=["ner", "textcat"])
         except OSError as err:
             if tn.params["autodownload"]:
                 try:
@@ -154,10 +157,10 @@ class Corpus:
         `Corpus`
         """
         object_cols = data.select_dtypes(include="object").columns
-        if not doc_col and object_cols.empty:
+        if doc_col is None and object_cols.empty:
             raise NoDocumentColumnException("No suitable document column.")
-        if not doc_col:
-            doc_col = object_cols[0]
+        if doc_col is None:
+            doc_col = str(object_cols[0])
         return cls(data.copy()[doc_col], lang=lang)
 
     @classmethod
@@ -502,7 +505,7 @@ class Corpus:
         tt = (
             pd.melt(
                 self.nlp.map(func).apply(pd.Series).reset_index(),
-                id_vars="label",
+                id_vars=["label"],
                 value_name="term",
             )
             .rename(columns={"variable": "n"})
@@ -630,7 +633,7 @@ def _ngrams(doc: list[str], n: int) -> list[str]:
     return [" ".join(t) for t in zip(*[doc[offset:] for offset in range(n)])]
 
 
-def _tf_idf(tidy_text: TidyText, sublinear: bool) -> TidyText:
+def _tf_idf(tidy_text: Union[pd.DataFrame, TidyText], sublinear: bool) -> TidyText:
     """Calculate term frequency/inverse document frequency."""
     if sublinear:
         tidy_text["tf"] = tidy_text["n"].map(_sublinear_scaling)
@@ -643,7 +646,7 @@ def _tf_idf(tidy_text: TidyText, sublinear: bool) -> TidyText:
         columns={"term_y": "idf"}
     )
     tt["term_weight"] = tt["tf"] * tt["idf"]
-    return tt[["term", "n", "term_weight"]]
+    return TidyText(tt[["term", "n", "term_weight"]])
 
 
 def _sublinear_scaling(n: Union[int, float]) -> float:
