@@ -2,29 +2,27 @@
 
 """Implements experimental features for formal concept analysis."""
 
+from abc import ABC, abstractmethod
 from typing import List, Tuple
+from warnings import warn
 
-import pandas as pd
 from toolz import memoize
 
 import textnets as tn
 
 
-class FormalContext:
+class FormalContext(ABC):
     """
-    Class providing experimental FCA features.
+    Abstract base class providing experimental FCA features.
 
     Textnets inherits methods from this class for treating its incidence matrix
     as a formal context.
-
-    Parameters
-    ----------
-    im : pandas.DataFrame
-        Incidence matrix of bipartite graph.
     """
 
-    def __init__(self, im: pd.DataFrame) -> None:
-        self.im = im
+    @property
+    @abstractmethod
+    def im(self):
+        pass
 
     @property
     def context(self) -> Tuple[List[str], List[str], List[List[bool]]]:
@@ -38,9 +36,14 @@ class FormalContext:
         # The incidence matrix is a "fuzzy formal context." We can binarize it
         # by using a cutoff. This is known as an alpha-cut. This feature is
         # experimental.
+        try:
+            from concepts import Context
+        except ImportError:
+            warn("Install textnets[fca] to use FCA features.")
+            raise
         crisp = self.im.applymap(lambda x: x >= alpha)
         reduced = crisp[crisp.any(axis=1)].loc[:, crisp.any(axis=0)]
         objects = reduced.index.tolist()
         properties = reduced.columns.tolist()
         bools = reduced.to_numpy().tolist()
-        return objects, properties, bools
+        return Context(objects, properties, bools)
