@@ -138,9 +138,14 @@ class TextnetBase(ABC):
         return self._partition
 
     @clusters.setter
-    def clusters(self, value: Union[ig.VertexClustering, dict[int, list[int]]]) -> None:
+    def clusters(
+        self,
+        value: Union[ig.VertexClustering, ig.VertexDendrogram, dict[int, list[int]]],
+    ) -> None:
         if isinstance(value, ig.VertexClustering):
             self._partition = value
+        elif isinstance(value, ig.VertexDendrogram):
+            self._partition = value.as_clustering()
         elif isinstance(value, dict):
             sorted_node_community_map = dict(sorted(value.items()))
             part = ig.VertexClustering(
@@ -151,6 +156,8 @@ class TextnetBase(ABC):
         elif isinstance(value, list):
             part = ig.VertexClustering(self.graph, membership=value)
             self._partition = part
+        else:
+            raise ValueError("No valid clusters supplied.")
 
     @clusters.deleter
     def clusters(self) -> None:
@@ -161,6 +168,30 @@ class TextnetBase(ABC):
         """Return modularity based on graph partition."""
         return self.graph.modularity(self.clusters, weights="weight")
 
+    @property
+    def cluster_strength(self) -> pd.Series:
+        """Weighted node degree within each cluster's subgraph."""
+        d = {}
+        for c in self.clusters:
+            subgraph = self.graph.subgraph(c)
+            subgraph_strength = subgraph.strength(weights="weight")
+            for n, strength in zip(c, subgraph_strength):
+                node = self.nodes[n]
+                d[node["id"]] = strength
+        return pd.Series(d).reindex(self.strength.index)
+
+    @property
+    def cluster_local_cc(self) -> pd.Series:
+        """Weighted local clustering coefficient within each cluster's subgraph."""
+        d = {}
+        for c in self.clusters:
+            subgraph = self.graph.subgraph(c)
+            subgraph_cc = subgraph.transitivity_local_undirected(weights="weight")
+            for n, cc in zip(c, subgraph_cc):
+                node = self.nodes[n]
+                d[node["id"]] = cc
+        return pd.Series(d).reindex(self.strength.index)
+
     def top_degree(self, n: int = 10) -> pd.Series:
         """
         Show nodes sorted by unweighted degree.
@@ -194,36 +225,36 @@ class TextnetBase(ABC):
         return self.strength.sort_values(ascending=False).head(n)
 
     def top_cluster_nodes(
-        self, n: int = 10, part: Optional[ig.VertexClustering] = None
-    ) -> pd.Series:
+        self, n: int = 10, rank_nodes_by: str = "cluster_strength"
+    ) -> pd.DataFrame:
         """
-        Show top nodes ranked by weighted degree per cluster.
+        Show top nodes per cluster, ranked by a chosen metric.
 
         Parameters
         ----------
         n : int, optional
             How many nodes to show per cluster (default: 10)
-        part : igraph.VertexClustering, optional
-            Partition to use (default: Leiden partition).
+        rank_nodes_by : str, optional
+            Metric to rank nodes within each cluster by (default:
+            cluster_strength).
 
         Returns
         -------
-        `pandas.Series`
-            Ranked nodes.
+        `pandas.DataFrame`
+            Clusters with representative nodes.
         """
-        if part is None:
-            part = self.clusters
         return (
             pd.DataFrame(
                 {
                     "nodes": self.nodes["id"],
-                    "strength": self.strength,
-                    "cluster": part.membership,
+                    "metric": getattr(self, rank_nodes_by),
+                    "cluster": self.clusters.membership,
                 }
             )
-            .sort_values("strength", ascending=False)
+            .sort_values("metric", ascending=False)
             .groupby("cluster")
-            .agg({"nodes": lambda x: ", ".join(x[:n])})["nodes"]
+            .agg({"nodes": lambda x: x[:n], "metric": len})
+            .rename(columns={"metric": "size"})
         )
 
     @decorate_plot
@@ -292,7 +323,7 @@ class Textnet(TextnetBase, FormalContext):
 
     Parameters
     ----------
-    data: TidyText
+    data: TidyText or IncidenceMatrix
         * DataFrame of tokens with per-document counts, as created by
           `Corpus.tokenized` `Corpus.ngrams`, and `Corpus.noun_phrases`.
         * An incidence matrix relating documents to terms.
@@ -302,6 +333,8 @@ class Textnet(TextnetBase, FormalContext):
     connected : bool, optional
         Keep only the largest connected component of the network (default:
         False).
+    remove_weak_edges : bool, optional
+        Remove edges with weights far below average (default: False).
     doc_attrs : dict of dict, optional
         Additional attributes of document nodes.
 
@@ -320,10 +353,11 @@ class Textnet(TextnetBase, FormalContext):
 
     def __init__(
         self,
-        data: TidyText,
+        data: Union[TidyText, IncidenceMatrix, pd.DataFrame],
         min_docs: int = 2,
         connected: bool = False,
-        doc_attrs: Optional[dict[str, dict[str, str]]] = None,
+        remove_weak_edges: bool = False,
+        doc_attrs: Optional[dict[str, dict[str, Any]]] = None,
     ) -> None:
         self._connected = connected
         self._doc_attrs = doc_attrs
@@ -333,12 +367,21 @@ class Textnet(TextnetBase, FormalContext):
             self._matrix = data
         elif isinstance(data, (TidyText, pd.DataFrame)):
             self._matrix = _im_from_tidy_text(data, min_docs)
+        if remove_weak_edges:
+            pairs: pd.Series[float] = self._matrix.stack()  # type: ignore
+            edge_weights: pd.Series[float] = pairs[pairs > 0]
+            iqr: float = edge_weights.quantile(0.75) - edge_weights.quantile(0.25)
+            cutoff: float = edge_weights.median() - 1.5 * iqr
+            if cutoff > 0:
+                self._matrix = IncidenceMatrix(
+                    self._matrix[self._matrix > cutoff].dropna(how="all").fillna(0)
+                )
 
     @cached_property
     def graph(self) -> ig.Graph:
         """Direct access to the underlying igraph object."""
         g = _graph_from_im(self._matrix)
-        if self._doc_attrs:
+        if self._doc_attrs is not None:
             for name, attr in self._doc_attrs.items():
                 g.vs[name] = [attr.get(doc) for doc in g.vs["id"]]
         if self._connected:
@@ -622,7 +665,7 @@ class Textnet(TextnetBase, FormalContext):
         return self.birank.sort_values(ascending=False).head(n)
 
     @cached_property
-    def clustering(self) -> pd.Series:
+    def bipartite_cc(self) -> pd.Series:
         """
         Calculate the unweighted bipartite clustering coefficient.
 
@@ -674,8 +717,21 @@ class ProjectedTextnet(TextnetBase):
     ----------
     graph : `igraph.Graph`
         Direct access to the igraph object.
+    m : `pandas.DataFrame`
+        The adjacency matrix.
     """
 
+    @cached_property
+    def m(self) -> pd.DataFrame:
+        """Matrix representation."""
+        d = self.graph.get_adjacency().data
+        m = np.array(d, dtype=float)
+        for i in self.edges.indices:
+            edge = self.graph.es[i]
+            source, target = edge.source, edge.target
+            m[source, target] = m[target, source] = edge["weight"]
+        return pd.DataFrame(m, index=self.nodes["id"], columns=self.nodes["id"])
+
     @cached_property
     def betweenness(self) -> pd.Series:
         """Weighted betweenness centrality."""
@@ -686,6 +742,13 @@ class ProjectedTextnet(TextnetBase):
         """Weighted closeness centrality."""
         return pd.Series(self.graph.closeness(weights="cost"), index=self.nodes["id"])
 
+    @cached_property
+    def harmonic(self) -> pd.Series:
+        """Weighted harmonic centrality."""
+        return pd.Series(
+            self.graph.harmonic_centrality(weights="cost"), index=self.nodes["id"]
+        )
+
     @cached_property
     def eigenvector_centrality(self) -> pd.Series:
         """Weighted eigenvector centrality."""
@@ -698,6 +761,19 @@ class ProjectedTextnet(TextnetBase):
         """Weighted PageRank centrality."""
         return pd.Series(self.graph.pagerank(weights="weight"), index=self.nodes["id"])
 
+    @property
+    def spanning(self) -> pd.Series:
+        """Textual spanning measure."""
+        if any(self.node_types):
+            warn("Textual spanning is only defined for document nodes.")
+        if not self.graph.is_connected():
+            warn(
+                "Graph is disconnected. "
+                "The textual spanning measure is not effective on disconnected graphs."
+            )
+        a = self.m.to_numpy()
+        return pd.Series(textual_spanning(a), index=self.nodes["id"])
+
     def top_betweenness(self, n: int = 10) -> pd.Series:
         """
         Show nodes sorted by betweenness.
@@ -730,6 +806,22 @@ class ProjectedTextnet(TextnetBase):
         """
         return self.closeness.sort_values(ascending=False).head(n)
 
+    def top_harmonic(self, n: int = 10) -> pd.Series:
+        """
+        Show nodes sorted by harmonic centrality.
+
+        Parameters
+        ----------
+        n : int, optional
+            How many nodes to show (default: 10)
+
+        Returns
+        -------
+        `pandas.Series`
+            Ranked nodes.
+        """
+        return self.harmonic.sort_values(ascending=False).head(n)
+
     def top_ev(self, n: int = 10) -> pd.Series:
         """
         Show nodes sorted by eigenvector centrality.
@@ -762,6 +854,22 @@ class ProjectedTextnet(TextnetBase):
         """
         return self.pagerank.sort_values(ascending=False).head(n)
 
+    def top_spanning(self, n: int = 10) -> pd.Series:
+        """
+        Show nodes sorted by textual spanning.
+
+        Parameters
+        ----------
+        n : int, optional
+            How many nodes to show (default: 10)
+
+        Returns
+        -------
+        `pandas.Series`
+            Ranked nodes.
+        """
+        return self.spanning.sort_values(ascending=False).head(n)
+
     def alpha_cut(self, alpha: float) -> ProjectedTextnet:
         """
         Return graph backbone.
@@ -829,7 +937,9 @@ class ProjectedTextnet(TextnetBase):
         return part
 
 
-def _im_from_tidy_text(tidy_text: TidyText, min_docs: int) -> IncidenceMatrix:
+def _im_from_tidy_text(
+    tidy_text: Union[TidyText, pd.DataFrame], min_docs: int
+) -> IncidenceMatrix:
     count = tidy_text.groupby("term").count()["n"]
     tt = (
         tidy_text.reset_index()
@@ -931,15 +1041,15 @@ def bipartite_rank(
     normalizer : string
         The normalizer to use: ``HITS``, ``CoHITS``, ``BGRM``, or ``BiRank``.
         See reference for details.
-    alpha : float
+    alpha : float, optional
         Damping factor for the rows and columns.
-    beta : float
+    beta : float, optional
         Damping factor for the rows and columns.
-    max_iter : int
+    max_iter : int, optional
         Maximum number of iterations to run before reaching convergence
         (default: -1, meaning iterate until the errors are within the
         specified tolerance).
-    tolerance : float
+    tolerance : float, optional
         Error tolerance when checking for convergence.
 
     Raises
@@ -1019,5 +1129,43 @@ def bipartite_rank(
     return pd.Series(np.append(d, p), index=net.nodes["id"])
 
 
+def textual_spanning(m: np.ndarray, alpha: float = 1.0) -> pd.Series:
+    """
+    Calculate textual spanning of documents in a similarity matrix.
+
+    Parameters
+    ----------
+    m : array
+        Document similarity matrix.
+    alpha : float, optional
+        Tuning parameter to change relevance of edge weights vs. document
+        counts in calculating the spanning score (default: 1.0).
+
+    Returns
+    -------
+    `numpy.array`
+        Each node's textual spanning measure.
+
+    Notes
+    -----
+    The textual spanning measure is not effective on disconnected graphs.
+
+    References
+    ----------
+    :cite:`Stoltz2019`
+    """
+    np.fill_diagonal(m, 0)
+    k = np.sum(m != 0, axis=1)
+    den = k * (m.sum(axis=1) / k**alpha)
+    ps = m.T / den[:, np.newaxis]
+    eps = np.zeros_like(ps)
+    eps[ps > 0] = ps[ps > 0] ** -1
+    ps2 = eps @ ps
+    sp = (ps + ps2) ** 2
+    csp = sp.sum(axis=1)
+    csp_norm = ((csp - csp.mean()) / csp.std(ddof=1)) * -1
+    return csp_norm
+
+
 class IncidenceMatrix(pd.DataFrame):
     """Matrix relating documents to terms."""
